#!/usr/bin/env python
# -*- coding: utf-8 -*-

class PageManager(object):
    """Simple singleton to keep generated page objects and re-use it on demand.
    Then we do not need to recreate a new object each time.

    *Note: Most of the inner page dynamic components should be recreated to
    ensure a state less rendering ;)*
    """

    class __Singleton:
        """Our singleton object.
        """

        def __init__(self):
            """Create the singleton object with a page dictonnary for the
            cache management.
            """
            self.page_dict = dict()

        def add_page(self, page_url, page):
            """Add a new page to the dictonnary with as key the url of the page
            and page as value.

            :param page_url: key for the dict cache
            :param page: page for the dict cache
            """
            self.page_dict[page_url] = page

        def get_page(self, page_url):
            """Returns the page in cache for the url key or None if page is not
            in cache.

            :param page_url: key for the dict cache
            :return: a page base instance or None
            :rtype: `bewype.core.component.PageBase`
            """
            # check page is in cache
            if self.has_page(page_url):
                return self.page_dict[page_url]
            # page not found
            else:
                return None

        def has_page(self, page_url):
            """Check the page is in cache for the given url.

            :return: True if page in cache, False otherwise
            :rtype: `bool`
            """
            return self.page_dict.has_key(page_url)

        def remove_page(self, page_url):
            """Removes a page from the cache. Because an admin user can modify
            page root content like layout, widget elements, Etc. after such
            changes a page should removed from the cache to be recreated on the
            next request.

            :param page_url: key for the dict cache
            """
            # check has page
            if self.has_page(page_url):
                # del page
                del self.page_dict[page_url]
            # not found                
            else:
                # do nothing
                pass

    # singleton instance
    instance = None

    def __new__(c):
        """Singleton new init.
        """
        # if doesn't already initialized
        if not PageManager.instance:
            # create a new instance
            PageManager.instance = PageManager.__Singleton()
        # return the manager object
        return PageManager.instance

