#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui & utils import
from bewype.core import ui, utils
# bewype component import
from bewype.core.component.component import Component
# bewype message import
from bewype.core.message import RpcResult

ACTION_TITLE_DICT = {
        "add": _("Add"),
        "cancel": _("Cancel"),
        "delete": _("Delete"),
        "update": _("Update")
        }

class ButtonBox(Component):

    def add_static_resources(self):
        # call parent
        ui.Widget.add_static_resources(self, file_=__file__)

    def get_value_key(self):
        return "button_box"

    def get_actions(self):
        return [
                "add",
                "update",
                "delete"
                ]

    def get_field(self, item):
        # init button model
        _model = list()
        # add setup buttons to the button box
        for _a in self.get_actions():
            _id = "%s%sButton" % (self.id_, utils.get_class_name(_a))
            _label = ACTION_TITLE_DICT[_a]
            _model.append({"id_": _id, "class_": self.class_, "label": _label})
        # create a button box
        return ui.ButtonBox(
                id_="%sButtonBox" % self.id_,
                class_=self.class_,
                model=_model,
                horizontal=True
                )

    def do_cancel(self, data, public=False):
        return RpcResult()

    def do_update(self, data, public=False):
        return RpcResult()

