#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os

# twisted import
from twisted.python import log

# nevow import
from nevow.i18n import _

# bewype ui import
from bewype.core import ui
# bewype component import
from bewype.core.component.form_field import FormField
# bewype resource import
from bewype.core.resource import Path, Resource

class FileEditor(FormField):

    def add_static_resources(self):
        FormField.add_static_resources(self, file_=__file__)

    def get_form_field(self, item):
        # get key for date entry
        _id = "%sTextView" % self.id_
        _value = self._get_value(item)
        # return the picker
        return ui.TextView(id_=_id, class_=self.class_, text=_value)

    def _get_path(self, item):
        # get file name
        if item is not None \
        and hasattr(item, self.get_value_key()):
            _filename = getattr(item, self.get_value_key())
        else:
            return None
        # get file path
        return Path(path=_filename, language_code=self.page.language_code,
                widget_element=self.widget_element)

    def _get_value(self, item):
        # get file path
        _p = self._get_path(item)
        if _p is None:
            return ""
        elif _p.get_type() == "img":
            return ""
        else:
            # return resource file content
            _content = Resource(_p.get_resource_path()).read()
            return "" if _content is False else _content

    def _set_value(self, item, content):
        # get file path
        _p = self._get_path(item)
        if _p is None:
            return False
        elif _p.get_type() == "img":
            return "Invalid file type 'img'!"
        else:
            _r_path = _p.get_resource_path()
            _s_path = _p.get_static_path()
            if _r_path is not None and _s_path is not None \
            and os.path.exists(_r_path) and os.path.exists(_s_path):
                # set resource file content and return update flag
                return Resource(_r_path).write(content) \
                        and Resource(_s_path).write(content)
            else:
                return True

