#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.python import log

# nevow import
from nevow import tags as T
from nevow.i18n import _

# bewype ui & utils import
from bewype.core import ui, utils
# bewype component import
from bewype.core.component.component import Component
# bewype meassge import
from bewype.core.message import RpcMessage, RpcResult

ACTION_TITLE_DICT = {
        "edit": _("Edit"),
        "delete": _("Delete"),
        "view": _("View")
        }

class List(Component):

    def add_static_resources(self):
        # call parent
        ui.Widget.add_static_resources(self, file_=__file__)

    def get_value_key(self):
        return "list"

    def get_actions(self):
        return [
                "delete",
                "edit"
                ]

    def get_render_func_dict(self):
        return {}

    def _get_render_func(self, func_cb_id):
        # get current func cb dict
        _render_func_dict = self.get_render_func_dict()
        # check has key
        if _render_func_dict.has_key(func_cb_id):
            return _render_func_dict[func_cb_id]
        else:
            return None

    def get_keys(self):
        """*Note* - Should be overriden by a child `class`.
        """
        raise NotImplementedError("Please implement this!")

    def _get_key_action_list(self):
        # get keys
        _k_list = self.get_keys()
        # update keys with action links
        map(lambda _a: _k_list.append("_%s" % _a), self.get_actions())
        # return the updated list
        return _k_list

    def get_titles(self):
        """*Note* - Should be overriden by a child `class`.
        """
        raise NotImplementedError("Please implement this!")

    def _get_title_dict(self):
        # get keys
        _k_list = self.get_keys()
        # get titles
        _titles = self.get_titles()
        # zip all
        _title_dict = dict(zip(_k_list, _titles))
        # update title dict var with action links
        for _a in self.get_actions():
            _title_dict["_%s" % _a] = ""
        # return the dict
        return _title_dict

    def _get_columns_args(self):
        """Can be overriden if necessary.
        """
        # get column dict and keys
        _title_dict = self._get_title_dict()
        # get keys + ations list
        _k_a_list = self._get_key_action_list()
        # add key id
        _k_a_list.insert(0, self._q.key_id)
        # init result
        _result_dict = dict()
        # update dict
        for _idx, _key in enumerate(_k_a_list):
            if _title_dict.has_key(_key):
                # prepare className
                _class = "%s%s" \
                        % (self.class_, utils.get_class_name(_key))
                # update kargs for links
                if _key == "_edit" or _key == "_delete" or _key == "_view":
                    # view specific idx
                    _actions = self.get_actions()
                    _ref_idx = _idx + _actions.index(_key.replace("_", ""))
                    # view specific args
                    _kargs = {
                            "text": _ref_idx,
                            "name": 0,
                            "href": _ref_idx + 1
                            }
                else:
                    # prepare kargs
                    _kargs = { "text": _idx }
                # set result dict row
                _result_dict[_key] = {
                        "title": _title_dict[_key],
                        "class_": _class,
                        "kargs": _kargs
                        }
            else:
                continue

        # return result            
        return _result_dict

    def _get_cell_value(self, row, key):
        # check if has a specific rendering method
        _render_func = self._get_render_func(key)
        # do render
        if key is None:
            return row
        # do render
        elif _render_func is not None:
            return _render_func(row)
        # get direct attribute
        elif key.find(".") == -1:
            return getattr(row, key)
        # try to get sub attribute
        else:
            try:
                # current object
                for _k in key.split("."):
                    row = getattr(row, _k)
                # result value should be in row now
                # !s sorry for that - thx py ;)
                return row
            # ouch!
            except:
                log.err("[w_listing_base] _get_cell_value - error: %s - %s" \
                        % (row, key))
                return None

    def _get_anchor_link(self):
        # link anchor
        return "#%s" % self.class_.lower()

    def get_view_link(self, row):
        return self._get_anchor_link()

    def _get_listing_row_values(self, key_list, action_list, row):
        # get db values
        _values = map(lambda _k: self._get_cell_value(row, _k), key_list)
        # set listing links
        for _a in action_list:
            # set link title
            _values.append(ACTION_TITLE_DICT[_a])
            # get link
            if _a is "view":
                _v = self.get_view_link(row)
            else:
                _v = self._get_anchor_link()
            # add link
            _values.append(_v)
        # return values for our treeview models
        return _values

    def _get_listing_model(self):
        # get all rows
        rows = self._q.get_all()
        # get keys
        _k_list = self.get_keys()
        # add key id
        _k_list.insert(0, self._q.key_id)
        # get current action list
        _action_list = self.get_actions()
        # parse values only
        return map(lambda _r:
                self._get_listing_row_values(_k_list, _action_list, _r), rows)

    def _add_column(self, treeview, column_args):
        # set cell renderer
        _cell_rend = ui.CellRendererText(class_=column_args["class_"])
        # set column
        _tvcolumn = ui.TreeViewColumn(column_args["title"])
        _tvcolumn.pack_start(_cell_rend)
        # args
        _kargs = column_args["kargs"]
        for _k in _kargs.keys():
            _tvcolumn.add_attribute(_cell_rend, _k, _kargs[_k])
        # add column to the treeview
        treeview.append_column(_tvcolumn)

    def get_field(self, item):
        #
        _model = self._get_listing_model()
        # create the TreeView using the query
        _id = "%sTreeView" % self.id_
        _treeview = ui.TreeView(id_=_id, class_=self.class_, model=_model,
                use_head=True)
        # get keys + actions list
        _k_a_list = self._get_key_action_list()
        # get values to render
        _columns_args = self._get_columns_args()
        # add columns according to the passed keys
        for _key in _k_a_list:
            self._add_column(_treeview, _columns_args[_key])
        # add the box
        return _treeview

    def do_reload(self, data, public=False):
        # get avatar id
        _avatarId = self.page.get_avatar_id()
        # prepare result
        _r = RpcResult()
        # not connected
        if _avatarId is None \
        and public is False:
            _r.error(_("Not connected!"))
            return _r
        else:
            # get keys + actions list
            _k_a_list = self._get_key_action_list()
            # get values to render
            _columns_args = self._get_columns_args()
            # assume None is the defatult header value
            _model = self._get_listing_model()
            # set result
            _r.success(_("OK"))
            _r.add("keyList", _k_a_list)
            _r.add("argsList", _columns_args)
            _r.add("model", _model)
            return _r

