#!/usr/bin/env python
# -*- coding: utf-8 -*-

# bewype data import
from bewype.core.data.yaml_base_loader import YamlBaseLoader
# bewype db import
from bewype.core.db import Query
# bewype utils import
from bewype.core.utils import get_key_name

class YamlDbLoader(YamlBaseLoader):
    """Load bewype widget db items from a yaml file. Nice way to bootstrap
    your application (users, languages, widgets, Etc.).

    Here is the expected yaml db file structure::

        config_name:
            OrmClassName:
                1:
                    key_name_1: value_1
                    key_name_2: value_2

    :see: `YamlBaseLoader`
    """

    def _exist(self, orm_class_name, id_value):
        """Checks if db item already exist in database.

        :param orm_class_name: orm class name for the db item to check.
        :param id_value: id of db item to check.

        :return: a boolean flag to True if already in db, False otherwise.
        :rtype: `boolean`

        :see: `Query`
        """
        # get config query
        _q = Query(orm_class_name)
        # get col name
        _col_id_name = "%s_id" % get_key_name(orm_class_name)
        # return exist result flag
        return _q.get_item(_col_id_name, id_value) is not None

    def load_section(self, config_key, section_key):
        """Prepare a dictionary with attributes and values to insert a new db
        item from yaml section cotent. Then the new item will be inserted to the
        current db session and commit.

        :param config_key: used as class name for the db item.
        :param section_key: used as id of db item.
        """
        # get value content
        _s_content = self.get_section_content(config_key, section_key)
        # check content not None and db item does not exist yet
        if _s_content is None \
        or self._exist(config_key, section_key):
            pass
        else:
            # init new query
            _q = Query(config_key)
            # get col name
            _col_id_name = "%s_id" % get_key_name(config_key)
            # force id
            _s_content[_col_id_name] = section_key
            # insert content
            _q.insert(_s_content)

    def load_value(self, config_key, section_key, value_key):
        """Not use for db loading!
        """
        pass
