#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os

# twisted import
from twisted.python import log

class Resource(object):
    """Gives simple resource like methode for a given path, ex.:
        - write
        - remove
        - Etc.
    """

    def __init__(self, path):
        """Creates the manager instance with the resource to manage.

        :param path: a resource path
        """
        # set the main path variable
        self.path = path

    def set_path(self, path):
        """
        """
        self.path = path

    def get_path(self):
        """
        """
        return self.path

    def read(self):
        """
        """
        if self.path is None \
        or not os.path.exists(self.path):
            return False
        else:
            try:
                # copy the data from key: userfile
                _f = open(self.path)
                _f.seek(0)
                _data = _f.read()
                _f.close()
                return _data
            except Exception, e:
                # ERROR
                log.err("[Resource] read - %s | e: " % (self.path, e))
                return False

    def write(self, data):
        """
        """
        if self.path is None \
        or data is None:
            return False
        else:
            try:
                # create non-existing folders
                _resource_dir, _f = os.path.split(self.path)
                # check dir exist
                if os.path.exists(_resource_dir):
                    pass
                # ensure dir
                else:
                    os.makedirs(_resource_dir)
                # copy the data from key: userfile
                _f = open(self.path, "wb")
                _f.write(data)
                _f.close()
                return True
            except Exception, e:
                # ERROR
                log.err("[Resource] write - %s | e: " % (self.path, e))
                return False

    def remove(self):
        """Checks first if the given path exist. If True the file or dir will be
        removed, otherwise nothing happen.

        Here is an example with an existing file::

            >>> _f = open('dummy.txt', 'wb')
            >>> _f.close()

        Existing file removal should returned True flag::

            >>> Resource('dummy.txt').remove()
            True

        Non-existing file removal should returned False::

            >>> Resource('dummy.txt').remove()
            False

        Also works for non-empty directory::

            >>> os.mkdir('test_dir')
            >>> _f = open('test_dir/dummy.txt', 'wb')
            >>> _f.close()

        Remove the non-empty directory::

            >>> Resource('test_dir').remove()
            True

        :return: a file or dir was removed flag
        :rtype: `bool`
        """
        # little check
        if self.path is None:
            pass
        # check the path exist
        elif os.path.exists(self.path):
            if os.path.isfile(self.path):
                try:
                    # do remove
                    os.remove(self.path)
                except Exception, e:
                    # ERROR
                    log.err("[resource] remove - e1: %s" % e)
                    # ERROR
                    return False
            elif os.path.isdir(self.path):
                try:
                    # remove files and dirs of the folder
                    for _f in os.listdir(self.path):
                        Resource(os.path.join(self.path, _f)).remove()
                    # remove empty dir at the end
                    os.rmdir(self.path)
                except Exception, e:
                    # ERROR
                    log.err("[resource] remove - e2: %s" % e)
                    # ERROR
                    return False
            else:
                pass
        else:
            pass
        # nothing to do
        return True

