#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os

# twisted import
from twisted.python import log

# genshi templating engine import
from genshi.template.base import Context
from genshi.template import MarkupTemplate, TextTemplate, TemplateLoader

# bewype web import
from bewype.core.resource import Path

TEMPLATING_TYPES = [
        "HTML",
        "TXT"
        ]

class Template(object):

    def __init__(self, tmpl_path):
        # set tmpl path
        self.tmpl_path = tmpl_path
        # init loader
        self.tmpl_loader = TemplateLoader(["."])

    def render(self, key_dict, file_ext=None):
        # ensure file ext using Path tool
        if file_ext is None:
            # get resource type dynamically from _file_path
            file_ext = Path(self.tmpl_path).get_ext()
        else:
            pass
        # get template obj
        if file_ext == "html":
            tmpl_cls = MarkupTemplate
        else:
            tmpl_cls = TextTemplate
        # load template
        _tmpl = self.tmpl_loader.load(self.tmpl_path, cls=tmpl_cls)
        # init context
        ctx_ = Context()
        ctx_.push(key_dict)
        # generate stream
        stream = _tmpl.generate(ctx_)
        # do render
        return stream.render()

    def write(self, key_dict, dst_path, file_ext=None):
        # ensure file ext using Path tool
        if file_ext is None:
            # get resource type dynamically from _file_path
            file_ext = Path(dst_path).get_ext()
        else:
            pass
        # get stream
        _stream = self.render(key_dict, file_ext=file_ext)
        if _stream is None:
            return False
        else:
            # get the server dir
            _dst_dir, _f = os.path.split(dst_path)
            # check dir exist or create it
            if not os.path.exists(_dst_dir):
                os.makedirs(_dst_dir)
            else:
                pass
            # open dest file
            _f = open(dst_path, "wb")
            # write content
            _f.write(_stream)
            # close it!
            _f.close()
            # result flag
            return True

