#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os, unittest
#
from ConfigParser import SafeConfigParser

# turbomail import
from turbomail.control import interface

# bewype config import
from bewype.core.config import Config
# bewype resource import
from bewype.core.email import Sender

class TestEmailSender(unittest.TestCase):

    def setUp(self):
        # create config object
        _config = SafeConfigParser()
        # set minimal value
        _config.add_section("email")
        _config.set("email", "mail_smtp_server", "dummy_server")
        _config.set("email", "mail_transport", "debug")
        # init config singleton
        Config(_config, force=True)

    def test_send_txt(self):
        # create a sender object
        _sender = Sender("admin@bewype.org")

        # send a simple email
        _sender.send("at@somebody.com", "subject", "content")
        # check nb of email sent
        _send_mails = interface.manager.transport.get_sent_mails()
        assert len(_send_mails) == 1, "Found: %s" % _send_mails

        # send a simple email -> email should not have been sent
        assert _sender.send(None, "subject", "content") is False
        # check nb of email sent -> email should not have been sent
        _send_mails = interface.manager.transport.get_sent_mails()
        assert len(_send_mails) == 1, "Found: %s" % _send_mails

        # send a simple email -> email should not have been sent
        assert _sender.send("at@somebody.com", None, "content") is False
        # check nb of email sent -> email should not have been sent
        _send_mails = interface.manager.transport.get_sent_mails()
        assert len(_send_mails) == 1, "Found: %s" % _send_mails

        # send a simple email -> email should not have been sent
        assert _sender.send("at@somebody.com", "subject", None) is False
        # check nb of email sent -> email should not have been sent
        _send_mails = interface.manager.transport.get_sent_mails()
        assert len(_send_mails) == 1, "Found: %s" % _send_mails

    def test_send_html(self):
        # prepare html content
        _content_html = """
<html>
    <head />
    <body>
        <h1>HTML Content</h1>
    </body>
</html
        """
        # create a sender object
        _sender = Sender("admin@bewype.org")
        # send a simple email
        _sender.send("at@somebody.com", "subject", "content",
                content_html=_content_html)
        # check nb of email sent
        _send_mails = interface.manager.transport.get_sent_mails()
        assert len(_send_mails) == 1, "Found: %s" % len(_send_mails)
        # check email contains html content
        assert _content_html in _send_mails[0], "Found: %s" % _send_mails[0]

    def test_send_attachment(self):
        # get resource dir
        _static_path = "bewype/core/tests/resource/img"
        _a_path = os.path.join(_static_path, "res_test.jpg")
        # prepare attachement list
        _a_list = [_a_path]
        # create a sender object
        _sender = Sender("admin@bewype.org")
        # send a simple email
        _sender.send("at@somebody.com", "subject", "content",
                attachment_list=_a_list)
        # check nb of email sent
        _send_mails = interface.manager.transport.get_sent_mails()
        assert len(_send_mails) == 1, "Found: %s" % len(_send_mails)
        # check email contains attachment name
        assert "res_test.jpg" in _send_mails[0]

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestEmailSender, "test_email_sender"))
    return suite

