#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os, unittest
#
from ConfigParser import SafeConfigParser

# bewype db import
from bewype.core.db import Query
# bewype component import
from bewype.core.component import Page
# bewype config import
from bewype.core.config import Config
# bewype resource import
from bewype.core.resource import Path

class TestPath(unittest.TestCase):

    def setUp(self):
        # create config object
        _config = SafeConfigParser()
        # set minimal value
        _config.add_section("app")
        _config.set("app", "language", "en")
        # init config singleton
        Config(config=_config, force=True)
        # remove all dummy items for each test
        Query("WidgetElement").delete_all()
        Query("Page").delete_all()

    def tearDown(self):
        # remove all dummy items after each test
        Query("WidgetElement").delete_all()
        Query("Page").delete_all()

    def test_build_app_path_no_widget_element(self):
        """
        """
        # css
        _path = Path("test.css")
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "test.css", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "css/test.css", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/css/test.css", "Found: %s" % _p

        # js
        _path = Path("test.js")
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "test.js", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "script/test.js", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/script/test.js", "Found: %s" % _p

        # img
        _path = Path("test.jpg")
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "test.jpg", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "img/test.jpg", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/img/test.jpg", "Found: %s" % _p

    def test_build_app_path_with_widget_element(self):
        """
        """
        # create widget element
        _q = Query("WidgetElement")
        # prepare insert dict
        _i_dict = {
                "widget_element_id": 1,
                "display_name": u"Name",
                "element_position": 0,
                "layout_id": 0,
                "widget_id": 0,
                }
        # insert first item
        _q.insert(_i_dict)
        # get widget element
        _w_e = _q.get_item("widget_element_id", 1)
        # css
        _path = Path("test.css", widget_element=_w_e)
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "ElementOne/test.css", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "ElementOne/css/test.css", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/ElementOne/css/test.css", "Found: %s" % _p

        # js
        _path = Path("test.js", widget_element=_w_e)
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "ElementOne/test.js", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "ElementOne/script/test.js", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/ElementOne/script/test.js", "Found: %s" % _p

        # img without page
        _path = Path("test.jpg", widget_element=_w_e)
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "ElementOne/test.jpg", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "ElementOne/img/test.jpg", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/ElementOne/img/test.jpg", "Found: %s" % _p

    def test_build_app_path_with_page(self):
        """
        """
        # create widget element
        _q = Query("WidgetElement")
        # prepare insert dict
        _i_dict = {
                "widget_element_id": 1,
                "display_name": u"Name",
                "element_position": 0,
                "layout_id": 0,
                "widget_id": 0,
                }
        # insert first item
        _q.insert(_i_dict)
        # get widget element
        _we_item = _q.get_item("widget_element_id", 1)
        # img with page
        _path = Path("test.jpg", language_code="en", widget_element=_we_item)
        # check
        _p = _path.build_app_path("", use_type=False)
        assert _p == "ElementOne/en/test.jpg", "Found: %s" % _p
        _p = _path.build_app_path("")
        assert _p == "ElementOne/img/en/test.jpg", "Found: %s" % _p
        _p = _path.build_app_path("base")
        assert _p == "base/ElementOne/img/en/test.jpg", "Found: %s" % _p

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPath, "test_path"))
    return suite

if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(suite())

