#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# nevow import
from nevow import loaders, tags as T

# bewype import
from bewype.core import utils
#
from bewype.core.ui.bin import Bin

# Button
class Button(Bin):

    def __init__(self, id_=None, class_=None,
            name=None, label=None, markup=None):
        # init parent
        Bin.__init__(self, id_=id_, class_=class_)
        # set main values
        self.name = name
        self.label = label
        self.markup = markup
        # init default values
        self.use_stock = False
        self.image = None

    def get_docFactory(self):
        return T.input(type="submit", id=self.id_, class_=self.class_,
                    name=self.name, value=self.label, alt=self.markup)

    def set_label(self, label):
        self.label = label

    def get_label(self):
        return self.label

    def set_image(self, image):
        self.image = image

    def get_image(self):
        return self.image

# ToggleButton
class ToggleButton(Button):

    def __init__(self, id_=None, class_=None,
            name=None, label=None, value=None, markup=None):
        # init parent
        Button.__init__(self, id_=id_, class_=class_,
                name=name, label=label, markup=markup)
        # set tag value
        self.value = value
        # mode = True   -> displayed with label
        # mode = False  -> display only the button 
        self.draw_indicator = True
        # activity flag
        self.is_active = False

    def get_docFactory(self, type="checkbox"):
        # set checked flag
        if self.is_active == True:
            _checked = "checked"
        else:
            _checked = None
        # do render
        if self.label is None \
        or self.draw_indicator is False:
            # return input only with markup
            return T.input(type=type, id=self.id_, class_=self.class_,
                        name=self.name, value=self.value, alt=self.markup,
                        checked=_checked)
        else:
            # prepare span with label
            _span = T.span(class_=self.class_, alt=self.markup)
            #
            _input = T.input(type=type, id=self.id_, class_=self.class_,
                        name=self.name, value=self.value, checked=_checked)
            # return span + input + label
            return _span[_input, self.label]

    def set_draw_indicator(draw_indicator):
        self.draw_indicator = draw_indicator

    def get_draw_indicator(self):
        return self.draw_indicator

    def set_active(self, is_active):
        self.is_active = is_active

    def get_active(self):
        return self.is_active

    def toggled(self):
        self.is_active = not self.is_active

# CheckButton
class CheckButton(ToggleButton):

    def __init__(self, id_=None, class_=None,
            name=None, label=None, value=None, markup=None):
        # init parent
        ToggleButton.__init__(self, id_=id_, class_=class_,
                name=name, label=label, value=value, markup=markup)

    def get_docFactory(self):
        return ToggleButton.get_docFactory(self, type="checkbox")

# RadioButton
class RadioButton(CheckButton):

    def __init__(self, id_=None, class_=None,
            name=None, label=None, value=None, markup=None, group=None):
        # init parent
        CheckButton.__init__(self, id_=id_, class_=class_,
                name=name, label=label, value=value, markup=markup)
        # set radio group
        if group is None:
            self.group = list()
        else:
            self.group = group.get_group()
            self.name = group.name
        # add self to the group
        self.group.append(self)

    def _updateNames(self):
        if self.name is None:
            #
            _name = utils.generate_key(32)
            # ensure all the names of the group are equal
            for _g in self.group:
                _g.name = _name
        else:
            pass

    def get_docFactory(self):
        # ensure name for the group
        self._updateNames()
        # render
        return ToggleButton.get_docFactory(self, type="radio")

    def get_group(self):
        return self.group

    def set_group(self, group):
        # set radio group
        self.group = group
        # add self to the group
        self.group.append(self)

