#!/usr/bin/env python
# -*- coding: utf-8 -*-

# nevow import
from nevow import loaders, tags as T

# bewype import
from bewype.core.ui import Box

def get_cell_content(model_row, do_html_rendering,
        cell_renderer, kargs, rend_func):
    # simple data return if no renderer
    if do_html_rendering == False:
        return model_row[kargs["text"]]
    else:
        pass
    # update rendering function
    if rend_func is None:
        pass
    else:
        # overide render func as require
        cell_renderer.get_docFactory = rend_func
    # update values
    for _k in kargs:
        if hasattr(cell_renderer, _k):
            # renderer type ex.: CellRendererText
            # model_row ex.: ["TextVal", "MarkupVal", True]
            # kargs ex.: ["text": 0, "markup": 1, "visible": 2]
            if kargs[_k] is None:
                setattr(cell_renderer, _k, "")
            else:
                # ensure not None value
                _value = model_row[kargs[_k]]
                if _value is None:
                    setattr(cell_renderer, _k, "")
                else:
                    setattr(cell_renderer, _k, _value)
        else:
            pass
    # return element
    return cell_renderer.get_docFactory()

class CellLayout(object):

    def __init__(self, cell_renderer=None, **kargs):
        # set the column renderer to the column renderer list
        if cell_renderer is None:
            # basic init
            self.cell_renderer_list = list()
            self.kargs_list = list()
            self.func_list = list()
        else:
            # first renderer with no value set
            self.cell_renderer_list = [cell_renderer]
            self.kargs_list = [kargs]
            self.func_list = [None]

    def pack_start(self, cell_renderer, expand=True):
        # update renderer list
        self.cell_renderer_list.insert(0, cell_renderer)
        # update karg for coherence reasons
        self.kargs_list.insert(0, dict())
        self.func_list.insert(0, None)

    def pack_end(self, cell_renderer, expand=True):
        # update renderer list
        self.cell_renderer_list.append(cell_renderer)
        # update karg for coherence reasons
        self.kargs_list.append(dict())
        self.func_list.append(None)

    def get_cell_renderers(self):
        return self.cell_renderer_list

    def add_attribute(self, cell_renderer, attribute, column_num):
        # get the renderer
        if hasattr(cell_renderer, attribute) \
        and cell_renderer in self.cell_renderer_list:
            # index in the list
            _index = self.cell_renderer_list.index(cell_renderer)
            # update attribute
            self.kargs_list[_index][attribute] = column_num
        else:
            pass

    def set_attributes(self, cell_renderer, **kargs):
        for _attr, _col in kargs.items():
            self.add_attribute(cell_renderer, _attr, _col)

    def set_cell_data_func(self, cell_renderer, func, func_data=None):
        # get the renderer
        if cell_renderer in self.cell_renderer_list:
            # index in the list
            _index = self.cell_renderer_list.index(cell_renderer)

            # prepare new func or not and do set
            if func_data is None:
                self.func_list[_index] = partial(func, func_data)
            else:
                self.func_list[_index] = func

