#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# nevow import
from nevow import tags as T

# bewype import
from bewype.core.ui.bin import Bin
from bewype.core.ui.cell_layout import get_cell_content, CellLayout
from bewype.core.ui.label import Label

def _get_option(cell_layout, class_, active_list, model_row):
    # we just retrieve the value without rendering
    _do_html_rendering = False
    # prepare function to prepare col elements
    _get_content_func = partial(
            get_cell_content,
            model_row, _do_html_rendering)
    # parse value
    _value = model_row[-1]
    # check if selected
    if active_list is not None \
    and _value in active_list:
        _selected = "selected"
    else:
        _selected = None
    # return nevow item for rendering
    return T.option(class_=class_, value=_value, selected=_selected)[
            map(
                _get_content_func,
                cell_layout.cell_renderer_list,
                cell_layout.kargs_list,
                cell_layout.func_list
                )
            ]

class ComboBox(Bin, CellLayout):

    def __init__(self, id_=None, class_=None,
            title=None, model=None, active_list=None,
            cell_renderer=None):
        # Init parents
        Bin.__init__(self, id_=id_, class_=class_)
        CellLayout.__init__(self, cell_renderer=None)
        # set main values
        self.title = title
        self.model = model
        # rendering values
        self.multiple = 1
        self.size = 0
        self.active_list = active_list

    def pack_start(self, cell_renderer, expand=True):
        CellLayout.pack_start(self, cell_renderer, expand=expand)

    def pack_end(self, cell_renderer, expand=True):
        CellLayout.pack_end(self, cell_renderer, expand=expand)

    def get_docFactory(self):
        """Something like:

        .. sourcecode:: xml

        <select size=1 name=SomeName>
        <option value="one">first</option>
        <option value="two">second</option>
        <option value="three">third</option>
        </select>

        """
        # update func
        _get_option_func = partial(
                _get_option, self, self.class_, self.active_list)
        # get options
        _options = map(_get_option_func, self.model)
        # want dropdown list
        if self.size == 0:
            # the html combo element
            _select = T.select(
                    id=self.id_,
                    class_=self.class_)[ _options ]
        # want combo
        else:
            # the html combo element
            _select = T.select(
                    id=self.id_,
                    class_=self.class_,
                    multiple=self.multiple,
                    size=self.size)[ _options ]
        # manage title
        if self.title is None:
            # no title return select only
            return _select
        else:
            # create label widget
            _label_widget = Label(
                    id="%sTitle" % self.id,
                    class_=self.class_,
                    text=self.title)
            # return all
            return T.div(class_=self.class_)[
                    _label_widget,
                    _select
                    ]

    def get_title(self):
        return self.title

    def set_title(self, title):
        self.title = title

    def set_model(self, model=None):
        self.model = None

    def get_model(self):
        return self.model

    def get_active_list(self):
        return self.active_list

    def set_active_list(self, active_list):
        self.active_list = active_list

    def set_multiple(self, multiple):
        self.multiple = multiple

    def get_multiple(self):
        return self.multiple

    def set_size(self, size):
        self.size = size

    def get_size(self):
        return self.size

