#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from functools import partial

# twisted imort
from twisted.python import log

# nevow import
from nevow import tags as T

# bewype import
from bewype.core.ui.cell_layout import get_cell_content
from bewype.core.ui.container import Container
from bewype.core.ui.label import Label

def _get_col_contents(class_, model_row, cell_layout):
    # set div for element
    _div = T.div(class_=class_)
    # we need rendering here
    _do_html_rendering = True
    # prepare function to prepare col elements
    _get_content_func = partial(
            get_cell_content, model_row, _do_html_rendering)
    #
    _elements = _div[
        map(
            _get_content_func,
            cell_layout.cell_renderer_list,
            cell_layout.kargs_list,
            cell_layout.func_list
            )
        ]
    #
    return T.td(class_=class_)[_elements]

# simple table update function use to build our 
def _get_treeview_tr(tree_view, model_row):
    # new function to update a specific cell
    _get_cell_func = partial(_get_col_contents,
            tree_view.class_, model_row)
    #
    return T.tr(class_=tree_view.class_)[
            map(_get_cell_func, tree_view.column_list)
            ]

def _get_treeview_th(class_, header_visible, column):
    # prepare th tag
    _th_tag = T.th(class_=class_)
    #
    if header_visible == True:
        _header_label = Label(class_=class_, text=column.title)
        return _th_tag[
                _header_label.get_docFactory()
                ]
    else:
        return _th_tag

# TreeView
class TreeView(Container):

    def __init__(self, id_=None, class_=None, model=None, use_head=False):
        # init parent
        Container.__init__(self, id_=id_, class_=class_)
        # set tree view model
        self.model = model
        # init tree view column list variable
        self.column_list = list()
        # assume None is the defatult header value
        self.headers_visible = list()
        self.use_head = True
        # init treview main vars
        self.selection = None
        #
        self.hadjustment = None
        self.vadjustment = None

    def get_docFactory(self):
        # init table element
        _table = T.table(id=self.id_, class_=self.class_)
        #
        if len(self.model) > 0:
            _head = T.thead(class_=self.class_)
            _body = T.tbody(class_=self.class_)
            # get tr rows
            _get_tr_func = partial(_get_treeview_tr, self)
            _body_rows = map(_get_tr_func, self.model)
            #
            if self.use_head is True:
                # get header row
                _get_th_func = partial(_get_treeview_th, self.class_)
                _row_cells = map(
                                _get_th_func,
                                self.headers_visible,
                                self.column_list
                                )
                _th_row = T.tr(class_=self.class_)[_row_cells]
                # tabe with headers
                return _table[_head[_th_row], _body[_body_rows]]
            else:
                # tabe without headers
                return _table[_body[_body_rows]]
        else:
            return _table

    def get_model(self):
        return self.model

    def set_model(self, model=None):
        self.model = model

    def get_selection(self):
        return self.selection

    def set_selection(self, selection):
        self.selection = selection

    def get_hadjustment(self):
        return self.hadjustment

    def set_hadjustment(self, adjustment):
        self.hadjustment = adjustment

    def get_vadjustment(self):
        return self.vadjustment

    def set_vadjustment(self, adjustment):
        self.vadjustment = adjustment

    def get_headers_visible(self):
        return self.headers_visible

    def set_headers_visible(self, headers_visible):
        # ensure value match with the current model
        if len(headers_visible) == len(self.column_list):
            self.headers_visible = headers_visible
        else:
            pass

    def append_column(self, column):
        # do add
        self.column_list.append(column)
        # header is visible by default
        self.headers_visible.append(True)

    def insert_column(self, column, position):
        # insert column
        self.column_list.insert(position, column)
        # header visible by default
        self.headers_visible.insert(position, True)

    def remove_column(self, column):
        # get column position
        _rm_position = self.column_list.index(column)
        # do remove
        self.column_list.pop(_rm_position)
        self.headers_visible.pop(_rm_position)

    def get_column(self, n):
        return self.column_list[n]

    def get_columns(self):
        return self.column_list

