#!/usr/bin/env python
# -*- coding: utf-8 -*-

# twisted import
from twisted.application.internet import TCPServer, SSLServer

# bewype import
from bewype.core.web import portal

def get_factory(resource_cls, use_guard=False, use_site=True):
    # Nevow import
    from nevow import appserver, guard
    #
    if use_guard is True:
        # init application realm
        _realm = portal.Realm(resource_cls)
        # init get portal
        _portal = portal.create_portal(_realm)
        # set resource
        _resource = guard.SessionWrapper(_portal)
    else:
        _resource = resource_cls()
    # init site resource
    if use_site is True:
        return appserver.NevowSite(resource=_resource)
    else:
        return _resource

def make_simple_app(port, resource_cls, application=None,
        use_guard=False, use_site=True, debug=False):
    # get factory
    _factory = get_factory(resource_cls,
            use_guard=use_guard, use_site=use_site)
    # init site
    if debug is True:
        from twisted.internet import reactor
        reactor.listenTCP(port, _factory)
        return None, None
    else:
        _http_site = TCPServer(port, factory=_factory)
        if application is None:
            return _http_site
        else:
            _http_site.setServiceParent(application)
            return None

def make_secure_app(port, resource_cls, application, ssl_context,
        use_guard=True, use_site=True, debug=False):
    # get factory
    _factory = get_factory(resource_cls,
            use_guard=use_guard, use_site=use_site)
    # init site
    if debug is True:
        from twisted.internet import reactor
        reactor.listenSSL(port, _factory, ssl_context)
    else:
        _https_site = SSLServer(port, _factory, ssl_context)
        if application is None:
            return _http_site
        else:
            _https_site.setServiceParent(application)
            return None

