#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
from datetime import datetime

# sqlalchemy import
from sqlalchemy.orm.exc import NoResultFound

#
from zope.interface import implements

# twisted import
from twisted.cred import checkers, credentials, error, portal
#
from twisted.internet import defer
from twisted.internet.threads import deferToThread
#
from twisted.python import log

# nevow import
from nevow import inevow

# bewype import
from bewype.core.db import model, Query

class DBChecker(object):

    implements(checkers.ICredentialsChecker)
    # The passwords are hashed in the database. We can support only
    # plaintext passwords received over the network.
    credentialInterfaces = (credentials.IUsernamePassword, )

    def _cbuserfromdb(self, user):
        """
        """
        return user

    def __get_user(self, username):
        """Private function that returns a username and hased password
        from the database based on a username.
        """
        # TODO: verify validity of the unicode() call
        # we send a unicode object here because SQLAlchemy wants it,
        # but we should make sure this casting will always work!
        _defered = deferToThread(model.User.by_user_name, unicode(username))
        _defered.addCallback(self._cbuserfromdb)
        return _defered

    def _update_user_status(self, auth_succeed, message=None, user_name=None):
        if user_name is None:
            # no commit == good commit :)
            _commit_result = True
        else:
            try:
                # get user
                _user = Query("User").get_item("user_name", user_name)
            except NoResultFound, e:
                # Oops!
                return defer.fail(error.UnauthorizedLogin("No User Item!"))
            # update last attempt time
            _user.last_attempt = datetime.now()
            # update user nb of attempts
            if auth_succeed is True:
                _user.nb_of_attempt = 0
            elif _user.nb_of_attempt is None:
                _user.nb_of_attempt = 1
            else:
                _user.nb_of_attempt += 1
            # do commit
            _commit_result = query_utils.do_commit()
        # return result
        if _commit_result is True:
            if auth_succeed is True:
                return defer.succeed(user_name)
            else:
                return defer.fail(error.UnauthorizedLogin(message))
        else:
            return defer.fail(error.UnauthorizedLogin(_commit_result))

    def _cb_validate_password(self, userobj, up):
        """Callback used when a user is found in our database, this user still
        needs to have a matching password with the one given by the would be
        user at the other end of the wire.
        """
        if userobj is None:
            return self._update_user_status(False, message="Invalid User!")
        elif userobj.validate_password(up.password):
            return self._update_user_status(True, user_name=up.username)
        else:
            return self._update_user_status(False, message="Bad password!",
                    user_name=up.username)

    def _cb_invaliduser(self, *args, **kwargs):
        """Callback used when the username is not found in our database. We
        return a failure.
        """
        return self._update_user_status(False, message="Bad password!")

    def requestAvatarId(self, creds):
        """
        """
        #
        _up = credentials.IUsernamePassword(creds, None)
        #
        _defered = self.__get_user(creds.username)
        _defered.addCallback(self._cb_validate_password, _up)
        _defered.addErrback(self._cb_invaliduser, _up)
        #
        if _up is not None:
            return _defered
        else:
            return defer.fail(error.UnauthorizedLogin("No such user"))

class Realm(object):

    implements(portal.IRealm)

    def __init__(self, resource_cls):
        self.resource_cls = resource_cls
        self.avatarId = None

    def requestAvatar(self, avatarId, mind, interface):
        if interface is inevow.IResource:
            # do web stuff
            if avatarId is checkers.ANONYMOUS:
                self.avatarId = None
                resc = self.resource_cls(self)
                return (inevow.IResource, resc, resc.logout)
            else:
                self.avatarId = avatarId
                resc = self.resource_cls(self)
                return (inevow.IResource, resc, resc.logout)

def create_portal(realm=None, perspective_cls=None, checker_cls=None):
    """Creates and returns a new new twisted portal for authentication and
    session management.
    """
    # init application portal
    if perspective_cls is None:
        _portal = portal.Portal(realm)
    else:
        _portal = perspective_cls(realm)
    # init application checker
    if checker_cls is None:
        _checker = DBChecker()
    else:
        _checker = checker_cls()
    # register users
    _portal.registerChecker(_checker)
    _portal.registerChecker(
            checkers.AllowAnonymousAccess(),
            credentials.IAnonymous)
    # return initialized portal
    return _portal

