
/**
 *
 */
BewypeJS.Rpc.Handler = new Class(
{
   /**
     *
     */
    initialize: function (rpcUrl, getParamsCallBack, resultCallBack,
                    errorCallBacks, onClickElmts, params)
    {
        //
        MochiKit.Base.bindMethods(this);

        // simple initialized flag
        this._initialized = false;

        // ensure values exist and are not null
        if(MochiKit.Base.isUndefinedOrNull(rpcUrl)
            || MochiKit.Base.isUndefinedOrNull(getParamsCallBack)
            || MochiKit.Base.isUndefinedOrNull(resultCallBack))
        {
            // confirms its not initialized
            this._initialized = false;
        }
        else
        {
            // set our RPC url
            this.rpcUrl = rpcUrl;
            // set our params callback
            this.getParamsCallBack = getParamsCallBack;
            this.resultCallBack = resultCallBack;
            this.errorCallBacks = errorCallBacks;
            //
            if(MochiKit.Base.isUndefinedOrNull(onClickElmts)
                || onClickElmts.length == 0)
            {
                MochiKit.Base.noop();
            }
            else
            {
                var _prepareFunc = MochiKit.Base.partial(
                        this._prepareCallbacks, params);
                // do prepare
                MochiKit.Base.map(_prepareFunc, onClickElmts);
            }

            // now it is
            this._initialized = true;
        }
    },

    /**
     *
     */
    _prepareCallbacks: function (params, onClickElmt)
    {
        if(MochiKit.Base.isUndefinedOrNull(onClickElmt))
        {
            MochiKit.Base.noop();
        }
        else
        {
            // prepare caller
            var _doCallFunc = MochiKit.Base.partial(
                    this.doCall, onClickElmt, params);
            // set event handler
            MochiKit.Signal.connect(
                    onClickElmt, "onclick", _doCallFunc);
        }
    },

    /**
     *
     */
    handleServerError: function (linkElement, params, err)
    {
        // ERROR
        MochiKit.Logging.logError(
                err.message + " (error #" + err.number + ")");
        // ERROR
        if (MochiKit.Base.isUndefinedOrNull(this.errorCallBacks))
        {
            this.resultCallBack(linkElement, params, err);
        }
        else
        {
            this.errorCallBacks(linkElement, params, err);
        }
    },

    /**
     *
     */
    handleServerResult: function (linkElement, params, rpcResult)
    {
        //res.responseText contains our result. Our CGI
        //returns it as a JS array inside a string
        //but just let MochiKit handle it for us
        resultDict = MochiKit.Async.evalJSONRequest(rpcResult);

        // provide result to the user callback
        this.resultCallBack(linkElement, params, resultDict);
    },

    /**
     * Remove param if start with _ letter.
     */
    __getCleanParams: function(paramsFrom, paramsTo, key)
    {
        if (MochiKit.Text.startsWith("_", key))
        {
            // pass
        }
        else
        {
            paramsTo[key] = paramsFrom[key];
        }
    },

    /**
     * force start rpc connection on click
     */
    doCall: function (linkElement, params)
    {
        /*
         *  params is something like:
         *  -------------------------
         *  {
         *      "username": "<uname>",
         *      "password": "<pwd>"
         *  };
         */

        // no params -> do nothing
        if(this._initialized == false
            || MochiKit.Base.isNull(this.rpcUrl)
            || MochiKit.Base.isNull(this.getParamsCallBack)
            || MochiKit.Base.isNull(this.resultCallBack))
        {
            return this.resultCallBack(linkElement, params, null);
        }
        else
        {
            // continue
            MochiKit.Base.noop();
        }

        var _rawContent = this.getParamsCallBack(linkElement, params);
        var _cloneContent = {};

        // ensure content
        if(MochiKit.Base.isUndefinedOrNull(_rawContent))
        {
            // ensure default empty value just for a call
            return this.resultCallBack(linkElement, params, null);
        }
        else
        {
            // prepare get clean params func
            var _getCleanParamsFunc = MochiKit.Base.partial(
                    this.__getCleanParams, _rawContent, _cloneContent);
            // remove junk params
            MochiKit.Base.map(
                    _getCleanParamsFunc,
                    MochiKit.Base.keys(_rawContent)
                    )
        }

        // format json content
        var _rpcContent = MochiKit.Base.queryString(_cloneContent);
        
        // DEBUG
        MochiKit.Logging.logDebug("[RpcHandler] doCall - rpcUrl: " + this.rpcUrl);
        MochiKit.Logging.logDebug("[RpcHandler] doCall - _rpcContent: " + _rpcContent);
        // DEBUG

        // prepare request
        var deferred = MochiKit.Async.doXHR(
                this.rpcUrl,
                {
                    method: "POST",
                    sendContent: _rpcContent,
                    headers: {
                        "Content-Type": "application/x-www-form-urlencoded"
                        }
                    }
                );

        // add callbacks
        deferred.addCallback(this.handleServerResult, linkElement, params);
        deferred.addErrback(this.handleServerError, linkElement, params);
        
        // DEBUG
        MochiKit.Logging.logDebug("[RpcHandler] doCall done!");
        // DEBUG
        
        // return deferred at the end
        return deferred;
    }
});

