
/**
 *
 */
BewypeJS.Ui.ButtonParser =
{
    parse: function (elementId)
    {
        var _params = BewypeJS.Ui.WidgetParser._parse(elementId);
        
        if(MochiKit.Base.isUndefinedOrNull(_params))
        {
            return null;
        }
        else
        {
            // return new button
            return new BewypeJS.Ui.Button(_params);
        }
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.WidgetParser.swap(elementId, widget);
    }
};

/**
 *
 */
BewypeJS.Ui.Button = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Bin,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);
        //
        this.name = BewypeJS.Utils.getVal(params, "name");
        this.label = BewypeJS.Utils.getVal(params, "label");
        this.markup = BewypeJS.Utils.getVal(params, "markup");
        this.stock = BewypeJS.Utils.getVal(params, "stock");
        //
        this.useStock = false;
        this.image = null;
    },

    getDocFactory: function ()
    {
        //
        this.attrs["type"] = "submit";
        this.attrs["name"] = this.name;
        this.attrs["value"] = this.label;
        this.attrs["alt"] = this.markup;
        // make it clean
        BewypeJS.Utils.Array.cleanDict(this.attrs);
        //
        return MochiKit.DOM.INPUT(this.attrs, null);
    },

    setLabel: function (label)
    {
        this.label = label;
    },

    getLabel: function ()
    {
        return this.label;
    },

    setUseStock: function (useStock)
    {
        this.useStock = useStock;
    },

    getUseStock: function ()
    {
        return this.useStock;
    },

    setImage: function (image)
    {
        this.image = image;
    },

    getImage: function ()
    {
        return this.image;
    }
});

/**
 *
 */
BewypeJS.Ui.ToggleButtonParser =
{
    _parseLabel: function (elementId)
    {
        // get element first
        var _inputElmt = MochiKit.DOM.getElement(elementId);
        var _spanElmt = _inputElmt.parentElement;

        if(!MochiKit.Base.isUndefinedOrNull(_spanElmt)
            && (_spanElmt.tagName == "span"
                || _spanElmt.tagName == "SPAN"))
        {
            // init params
            var _params = {};
            // get label
            _params["label"] = _spanElmt.innerText;
            // get markup
            _params["markup"] = BewypeJS.Utils.getAttrVal(_spanElmt, "alt");
            //
            return _params;
        }
        else
        {
            return null;
        }
    },

    _parseInput: function (elementId)
    {
        // get element first
        var _inputElmt = MochiKit.DOM.getElement(elementId);

        if(MochiKit.Base.isUndefinedOrNull(_inputElmt)
            && (_inputElmt.tagName == "input"
                || _inputElmt.tagName == "INPUT"))
        {
            return null;
        }
        else
        {
            // init params
            var _params = {};
            // get id
            _params["id"] = _inputElmt.id;
            // get class
            _params["class"] = _inputElmt.className;
            // get name
            _params["name"] = _inputElmt.name;
            // get stock    -| not used yet
            _params["stock"] = null;

            // return new button
            return _params;
        }
    },

    parse: function (buttonType, elementId)
    {
        // get params from html element
        var _labelParams = BewypeJS.Ui.ToggleButtonParser._parseLabel(elementId);
        var _inputParams = BewypeJS.Ui.ToggleButtonParser._parseInput(elementId);

        // init new object params
        var _params = {};

        if(MochiKit.Base.isUndefinedOrNull(_labelParams))
        {
            _params = _inputParams;
        }
        else
        {
            if(MochiKit.Base.isUndefinedOrNull(_inputParams))
            {
                // nothing to build
                return null;
            }
            else
            {
                BewypeJS.Utils.Array.concatDict(_labelParams, _params);
                BewypeJS.Utils.Array.concatDict(_inputParams, _params);
            }
        }

        //
        if(buttonType == "checkbox")
        {
            return new BewypeJS.Ui.CheckButton(_params);
        }
        else
        {
            return new BewypeJS.Ui.RadioButton(_params);
        }
    },

    swap: function (elementId, widget)
    {
        // get element first
        var _inputElmt = MochiKit.DOM.getElement(elementId);
        var _spanElmt = _inputElmt.parentElement;

        //
        if(MochiKit.Base.isUndefinedOrNull(_spanElmt))
        {
            if(MochiKit.Base.isUndefinedOrNull(_inputElmt))
            {
                // do nothing
                MochiKit.Base.noop();
            }
            else
            {
                // do swap
                MochiKit.DOM.swapDOM(_inputElmt, widget.getDocFactory());
            }
        }
        else
        {
            // do swap
            MochiKit.DOM.swapDOM(_spanElmt, widget.getDocFactory());
        }
    }
};

/**
 * ToggleButton
 */
BewypeJS.Ui.ToggleButton = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Button,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);
        // set tag value
        this.value = BewypeJS.Utils.getVal(params, "value");
        // mode = True   -> displayed with label
        // mode = False  -> display only the button
        this.drawIndicator = true;
        //
        this.isActive = false;
    },

    _getDocFactory: function (type)
    {
        // set checked flag
        var _checked = null;
        if(this.isActive == true)
        {
            this.attrs["checked"] = "checked";
        }
        else
        {
            // nothing
            MochiKit.Base.noop();
        }

        // set input attrs
        this.attrs["type"] = type;
        this.attrs["name"] = this.name;
        this.attrs["value"] = this.value;

        if(MochiKit.Base.isUndefinedOrNull(this.label)
            || this.drawIndicator == false)
        {
            // manage markup only
            this.attrs["alt"] = this.markup;
            // return input only
            return MochiKit.DOM.INPUT(this.attrs, null);
        }
        else
        {
            // span manage markup for the component
            var _spanAttrs = {
                "class": BewypeJS.Utils.getVal(this.attrs, "class"),
                "alt": this.markup
            };

            // make it clean
            BewypeJS.Utils.Array.cleanDict(this.attrs);
            BewypeJS.Utils.Array.cleanDict(_spanAttrs);

            // set input value
            var _input = MochiKit.DOM.INPUT(this.attrs, null);

            // return span + input + label
            return MochiKit.DOM.SPAN(_spanAttrs, [_input, this.label]);
        }
    },

    setMode: function (drawIndicator)
    {
        this.drawIndicator = drawIndicator;
    },

    getMode: function ()
    {
        return this.drawIndicator;
    },

    set_active: function (isActive)
    {
        this.isActive = isActive;
    },

    get_active: function ()
    {
        return this.isActive;
    },

    toggled: function ()
    {
        this.isActive = !this.isActive;
    }
});

/**
 *
 */
BewypeJS.Ui.CheckButtonParser =
{
    parse: function (elementId)
    {
        return BewypeJS.Ui.ToggleButtonParser.parse("checkbox", elementId);
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.ToggleButtonParser.swap(elementId, widget);
    }
};

/**
 * CheckButton
 */
BewypeJS.Ui.CheckButton = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.ToggleButton,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);
    },

    getDocFactory: function (params)
    {
        // init parent
        return this._getDocFactory("checkbox");
    }
});

/**
 *
 */
BewypeJS.Ui.RadioButtonParser =
{
    parse: function (elementId)
    {
        return BewypeJS.Ui.ToggleButtonParser.parse("radio", elementId);
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.ToggleButtonParser.swap(elementId, widget);
    }
};

/**
 * RadioButton
 */
BewypeJS.Ui.RadioButton = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.CheckButton,    

    initialize: function (params)
    {
        // init parent
        this.parent(params);

        // set radio group
        var _button = BewypeJS.Utils.getVal(params, "group");
        if(MochiKit.Base.isUndefinedOrNull(_button))
        {
            this.group = new Array();
        }
        else
        {
            this.group = _button.getGroup();
            this.name = _button.name;
        }
        // add self to the group
        this.group.push(this);
    },

    _updateNames: function ()
    {
        if(MochiKit.Base.isUndefinedOrNull(this.name))
        {
            //
            _name = BewypeJS.Utils.generateKey(32);
            // ensure all the names of the group are equal
            _setNameFunc = MochiKit.Base.partial(
                    BewypeJS.Utils._setName, _name);
            // update
            MochiKit.Base.map(_setNameFunc, this.group);
        }
        else
        {
            MochiKit.Base.noop();
        }
    },

    getDocFactory: function (params)
    {
        //
        this._updateNames();
        // init parent
        return this._getDocFactory("radio");
    },
    
    getName: function ()
    {
        return this.name;
    },

    setName: function (name)
    {
        // set radio name
        this.name = name;
    },
    
    getGroup: function ()
    {
        return this.group;
    },

    setGroup: function (group)
    {
        if(MochiKit.Base.isUndefinedOrNull(group))
        {
            MochiKit.Base.noop();
        }
        else
        {
            // set radio group
            this.group = group;
            // add self to the group
            this.group.push(this);
        }
    }
});

