/**
 *
 */
BewypeJS.Ui.ComboBoxParser =
{
    parse: function (elementId)
    {
        // common params parsing
        var _params = BewypeJS.Ui.WidgetParser._parse(elementId);
        
        if(MochiKit.Base.isUndefinedOrNull(_params))
        {
            return null;
        }
        else
        {
            // return the new combo widget
            return new BewypeJS.Ui.ComboBox(_params);
        }
    },

    swap: function (elementId, widget)
    {
        BewypeJS.Ui.WidgetParser.swap(elementId, widget);
    }
};

/**
 *
 */
function getComboContents(cellLayout, classCb, activeList, modelRow)
{
    // we just retrieve the value without rendering
    var _doHtmlRendering = false;

    // prepare function to prepare col elements
    var _getContentFunc = partial(
            BewypeJS.Ui.Utils.getCellContent, modelRow, _doHtmlRendering);

    // parse value
    var _value = modelRow[modelRow.length - 1];

    // prepare attrs
    var _attrs = {
        "class": classCb,
        "value": _value,
    };

    if (MochiKit.Base.isUndefinedOrNull(activeList)
        || MochiKit.Base.isUndefinedOrNull(_value)
        || MochiKit.Base.findValue(activeList, _value) == -1)
    {
        MochiKit.Base.noop();
    }
    else
    {
        // add selected value
        _attrs["selected"] = "selected";
    }

    // return nevow item for rendering
    return MochiKit.DOM.OPTION(
            _attrs,
            MochiKit.Base.map(
                _getContentFunc,
                cellLayout.cellRendererList,
                cellLayout.kargsList,
                cellLayout.funcList
                )
            );
}

/**
 * ComboBox
 */
BewypeJS.Ui.ComboBox = new Class(
{
    // parent class
    Extends: BewypeJS.Ui.Bin,

    initialize: function (params)
    {
        // init parent
        this.parent(params);

        // cell layout wrapper instead of multiple heritage
        this.cellLayout = new BewypeJS.Ui.CellLayout(params);
        
        // set main values
        this.title = BewypeJS.Utils.getVal(params, "title");
        this.model = BewypeJS.Utils.getVal(params, "model");

        // rendering values
        this.multiple = 1;
        this.size = 0;
        this.activeList = BewypeJS.Utils.getVal(params, "activeList");
    },

    /**
     * ex.:
     *  <select size=1 name=SomeName>
     *      <option value="one">first</option>
     *      <option value="two">second</option>
     *      <option value="three">third</option>
     *  </select>
     */
    getDocFactory: function ()
    {
        // get current values
        var _id = BewypeJS.Utils.getVal(this.attrs, "id");
        var _class = BewypeJS.Utils.getVal(this.attrs, "class");
        var _multiple = BewypeJS.Utils.getVal(this.attrs, "multiple");

        // update func
        var getComboContentsFunc = partial(
            getComboContents, this.cellLayout, _class, this.activeList);

        //
        var _attrs = {
            "id": _id,
            "class": _class,
            };

        // manage single or multiple select
        if(this.size == 0)
        {
            MochiKit.Base.noop();
        }
        else
        {
            _attrs["size"] = this.size;
            _attrs["multiple"] = this.multiple;
        }

        // ensure valid model value for mapping
        var _model = null;
        if(MochiKit.Base.isUndefinedOrNull(this.model))
        {
            _model = [];
        }
        else
        {
            // ok
            _model = this.model;
        }

        // the html combo element
        var _select = MochiKit.DOM.SELECT(
                _attrs,
                MochiKit.Base.map(getComboContentsFunc, _model)
                );

        if(MochiKit.Base.isUndefinedOrNull(this.title))
        {
            return _select;
        }
        else
        {
            // div class
            var _params = {
                    "id": _id + "Title",
                    "class": _class,
                    "text": this.title
                };

            // title
            var _title_widget = new BewypeJS.Ui.Label(_params);

            // factory
            return MochiKit.DOM.DIV(
                    { "class": _class },
                    [
                        _title_widget.getDocFactory(),
                        _select
                        ]
                    )
        }
    },

    getTitle: function ()
    {
        return this.title;
    },

    setTitle: function (title)
    {
        this.title = title;
    },

    setModel: function (model)
    {
        this.model = model;
    },

    getModel: function ()
    {
        return this.model;
    },

    setActiveList: function (activeList)
    {
        this.activeList = activeList;
    },

    getActiveList: function ()
    {
        return this.activeList;
    },

    setMultiple: function (multiple)
    {
        this.multiple = multiple;
    },

    getMultiple: function ()
    {
        return this.multiple;
    },

    setSize: function (size)
    {
        this.size = size;
    },

    getSize: function ()
    {
        return this.size;
    },

    /**
     ******************DUMMY LAYOUT WRAPPING******************
     */

    /**
     *
     */
    packStart: function (cellRenderer, expand)
    {
        this.cellLayout.packStart(cellRenderer, expand);
    },
    
    /**
     *
     */
    packEnd: function (cellRenderer, expand)
    {
        this.cellLayout.packEnd(cellRenderer, expand);
    },

    /**
     *
     */
    clear: function ()
    {
        this.cellLayout.clear();
    },

    /**
     *
     */
    getCellRenderers: function ()
    {
        this.cellLayout.getCellRenderers();
    },

    /**
     *
     */
    addAttribute: function (cellRenderer, attribute, columnNum)
    {
        this.cellLayout.addAttribute(cellRenderer, attribute, columnNum);
    },

    /**
     *
     */
    setAttributes: function (cellRenderer, kargsList)
    {
        this.cellLayout.setAttributes(cellRenderer, kargsList);
    },

    /**
     *
     */
    setCellDataFunc: function (cellRenderer, func, funcData)
    {
        this.cellLayout.setCellDataFunc(cellRenderer, func, funcData);
    }

    /**
     ******************DUMMY LAYOUT WRAPPING******************
     */
});

