
/**
 *
 */
BewypeJS.Utils.Array =
{
    /**
     * Dummy pop function for easy mapping.
     */
    _doPop: function (list, item)
    {
        list.pop();
    },

    /**
     * Dummy push function for easy list mapping.
     */
    _doPush: function (list, item)
    {
        list.push(item);
    },

    /**
     * Dummy set function for easy dict mapping.
     */
    _doSet: function (dict, key, value)
    {
        dict[key] = value;
    },

    _doClean: function (dict, key)
    {
        if(MochiKit.Base.isUndefinedOrNull(dict[key]))
        {
            // remove empty value
            delete dict[key];
        }
        else
        {
            // do nothing
            MochiKit.Base.noop();
        }
    },

    cleanDict: function (dict)
    {
        //
        var _doCleanFunc = MochiKit.Base.partial(
                BewypeJS.UtilsArray._doClean, dict);
        //
        MochiKit.Base.map(
                _doCleanFunc,
                MochiKit.Base.keys(dict)
                );
    },

    /**
     * Empty a list and keep it, permit not to create
     * a new one and return it.
     */
    clearList: function (list)
    {
        // create dummy list to iter
        var dummyList = MochiKit.Base.map(MochiKit.Base.noop(), list);
        // ensure pop mapping
        var popFunc = MochiKit.Base.partial(BewypeJS.UtilsArray._doPop, list);
        // pop everything
        MochiKit.Base.map(popFunc, dummyList);
    },

    /**
     * Concat everything in the current not a new list.
     */
    concatList: function (listSrc, listToConcat)
    {
        // ensure push mapping
        var pushFunc = MochiKit.Base.partial(
                BewypeJS.UtilsArray._doPush, listToConcat);
        // push everything
        MochiKit.Base.map(pushFunc, listSrc);
    },

    /**
     * Concat everything in the current not a new list.
     */
    concatDict: function (dictSrc, dictToConcat)
    {
        // ensure set mapping
        var setFunc = MochiKit.Base.partial(
                BewypeJS.UtilsArray._doSet, dictToConcat);
        // set everything
        MochiKit.Base.map(
                setFunc,
                MochiKit.Base.keys(dictSrc),
                MochiKit.Base.values(dictSrc)
                );
    },

    /**
     * Insert an item at the asked position and update
     * the list passed in parameter.
     */
    insertToArray: function (list, item, position)
    {
        // separate for insert
        var partLeft = list.slice(0, position);
        var partRight = list.slice(position);
        // reset current list - just in case
        BewypeJS.UtilsArray.clearList(list);
        // rebuild list
        BewypeJS.UtilsArray.concatList(partLeft.concat([item], partRight), list);
    },

    /**
     * Remove an item if found and update
     * the list passed in parameter.
     *
     * Return the position of the removed index -> useful.
     */
    removeFromArray: function (list, item)
    {
        //
        var position = list.indexOf(item);

        if (position == -1) {
            MochiKit.Base.noop();
        }
        else
        {
            // separate for insert
            var partLeft = list.slice(0, position);
            var partRight = list.slice(position + 1);
            // rebuild list
            BewypeJS.UtilsArray.concatList(partLeft.concat(partRight), list);
        }
        // 
        return position;
    },

    /**
     * Remove an item at the position passed in parameter.
     */
    removeFromArrayAtPosition: function (list, position)
    {
        if (position == -1) {
            MochiKit.Base.noop();
        }
        else
        {
            // separate for insert
            var partLeft = list.slice(0, position);
            var partRight = list.slice(position + 1);
            // rebuild list
            BewypeJS.UtilsArray.concatList(partLeft.concat(partRight), list);
        }
    }
};
