/**
 *
 */
BewypeJS.Utils.Widget =
{

    _checkersGroups: {},
    _uploadersGroups: {},
    _componentsGroups: {},

    deferredDict: {},

    /**
     *
     */
    getTypeName: function(fieldType)
    {
        if(fieldType == "entry"
            || fieldType == "email")
        {
            return "Entry";
        }
        else if(fieldType == "password")
        {
            return "Password";
        }
        else if(fieldType == "file")
        {
            return "File";
        }
        else if(fieldType == "label")
        {
            return "Label";
        }
        else if(fieldType == "textview")
        {
            return "Textview";
        }
        else if(fieldType == "checkbox"
                || fieldType == "radiobox")
        {
            return "ButtonBox";
        }
        else if(fieldType == "combobox")
        {
            return "ComboBox";
        }
        else
        {
            return "";
        }
    },

    /**
     *
     */
    removeElement: function (elmtId)
    {
        // get current deferred
        var _def = BewypeJS.Utils.getVal(
                BewypeJS.Utils.Widget.deferredDict, elmtId);
        if(MochiKit.Base.isUndefinedOrNull(_def))
        {
            // nothing to cancel
        }
        else
        {
            // cancel deferred first
            _def.cancel();
            // remove deferred from dict
            delete BewypeJS.Utils.Widget.deferredDict[elmtId];
        }
        // get element
        var _elmt = MochiKit.DOM.getElement(elmtId);
        // if no element found
        if(MochiKit.Base.isUndefinedOrNull(_elmt))
        {
            // do nothing
            MochiKit.Base.noop();
        }
        else
        {
            // remove element
            swapDOM(_elmt, null);
        }
    },

    /**
     * Common method to append a message corresponding with a field.
     *
     * First get the parent of the field then append the message according
     * to the message type and values passed in parameters.
     */
    appendMessage: function (elmtToAppend, msgType, message, insertAfter)
    {
        // init parent value
        var _parentElmt = null;
        // ensure element
        elmtToAppend = MochiKit.DOM.getElement(elmtToAppend);
        //
        if (elmtToAppend.tagName.toLowerCase() == "div")
        {
            _parentElmt = elmtToAppend;
        }
        else
        {
            // try to get the first div
            _parentElmt = MochiKit.DOM.getFirstParentByTagAndClassName(
                    elmtToAppend,
                    tagName="div",
                    className=elmtToAppend.className);
        }

        if(MochiKit.Base.isUndefinedOrNull(_parentElmt))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Validators] appendMessage - no element for MESSAGE!");
            MochiKit.Logging.logDebug(
                    "[Validators] appendMessage - elmtToAppend.tagName: "
                    + elmtToAppend.tagName
                    );
            // DEBUG
        }
        else
        {
            // prepare element div            
            var _elmtId = elmtToAppend.id + msgType;
            // first remove previous warning
            BewypeJS.Utils.Widget.removeElement(_elmtId)
            // prepare warning
            var _infoElmt = MochiKit.DOM.DIV(
                        {
                            "id": _elmtId,
                            "class": elmtToAppend.className
                        },
                        message
                    );
            // append warning
            if(MochiKit.Base.isUndefinedOrNull(insertAfter))
            {
                MochiKit.DOM.insertSiblingNodesBefore(_parentElmt, _infoElmt);
            }
            else
            {
                MochiKit.DOM.insertSiblingNodesAfter(_parentElmt, _infoElmt);
            }
            // do later call fade mecanism
            var _def = MochiKit.Async.callLater(3, BewypeJS.Utils.Widget.removeElement, _elmtId);
            // add to the current defered dict
            BewypeJS.Utils.Widget.deferredDict[_elmtId] = _def
        }
    },

    //// MANAGE GROUP LIST [CHECKERS | UPLOADERS | COMPONENTS]

    _updateComponentId: function(itemId, component)
    {
        component._updateItemId(itemId);
    },

    getCheckers: function (groupId)
    {
        var _list = BewypeJS.Utils.getVal(
                BewypeJS.Utils.Widget._checkersGroups, groupId);
        //
        if (MochiKit.Base.isUndefinedOrNull(_list))
        {
            BewypeJS.Utils.Widget._checkersGroups[groupId] = [];
            return [];
        }
        else
        {
            return _list
        }
    },

    _addCheckerToTheGroup: function (groupId, checker)
    {
        // do add
        var _checkers = BewypeJS.Utils.Widget.getCheckers(groupId);
        // ensure list
        BewypeJS.Utils.Widget._checkersGroups[groupId].push(checker);
    },

    getUploaders: function (groupId)
    {
        _list = BewypeJS.Utils.getVal(
                BewypeJS.Utils.Widget._uploadersGroups, groupId);
        //
        if (MochiKit.Base.isUndefinedOrNull(_list))
        {
            BewypeJS.Utils.Widget._uploadersGroups[groupId] = [];
            return [];
        }
        else
        {
            return _list
        }
    },

    _addUploaderToTheGroup: function (groupId, uploader)
    {
        // ensure list
        var _uploaders = BewypeJS.Utils.Widget.getUploaders(groupId);
        // do add
        BewypeJS.Utils.Widget._uploadersGroups[groupId].push(uploader);
    },

    getComponents: function (groupId)
    {
        _list = BewypeJS.Utils.getVal(
                BewypeJS.Utils.Widget._componentsGroups, groupId);
        //
        if (MochiKit.Base.isUndefinedOrNull(_list))
        {
            BewypeJS.Utils.Widget._componentsGroups[groupId] = [];
            return [];
        }
        else
        {
            return _list
        }
    },

    _addComponentToTheGroup: function (groupId, widget)
    {
        // ensure list
        var _components = BewypeJS.Utils.Widget.getComponents(groupId);
        // do add
        BewypeJS.Utils.Widget._componentsGroups[groupId].push(widget);
    },

    /**
     *
     */
    updateGroups: function (groupId, widget)
    {
        var _managerList = BewypeJS.Utils.getVal(
                BewypeJS.Utils.Widget.wigdetManagers, groupId);
        //
        if (MochiKit.Base.findValue(widget.actionList, "Check") != -1)
        {
            BewypeJS.Utils.Widget._addCheckerToTheGroup(groupId, widget);
        }
        else if (MochiKit.Base.findValue(widget.actionList, "Upload") != -1)
        {
            BewypeJS.Utils.Widget._addUploaderToTheGroup(groupId, widget);
        }
        else
        {
            BewypeJS.Utils.Widget._addComponentToTheGroup(groupId, widget);
        }
    }
}

