/**
*
*/
BewypeJS.Validators = {

    /**
     *
     */
    validateEmptyBox: function (elmtToValidate, boxType)
    {
        if(MochiKit.Base.isUndefinedOrNull(elmtToValidate))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Validators] validateEmptyBox - elmtToValidate is NULL for: "
                    + elmtToValidate.id);
            // DEBUG
            return false;
        }
        else
        {
            // continue
            MochiKit.Base.noop();
        }

        // init value to check
        var _values = null;
        var _nbOfItems = 0;

        // do check
        if(boxType == "checkbox"
            || boxType == "radiobox")
        {
            var _activeResult = BewypeJS.UtilsBox.getButtonBoxActive(elmtToValidate);
            _values = _activeResult[0];
            _nbOfItems = _activeResult[1];
        }
        else if(boxType == "combobox")
        {
            var _activeResult = BewypeJS.UtilsBox.getComboBoxActive(elmtToValidate);
            _values = _activeResult[0];
            _nbOfItems = _activeResult[1];
        }
        else
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Validators] validateNotEmpty - boxType unkonwn: "
                    + boxType);
            // DEBUG
            return false
        }

        if (_values instanceof Array)
        {
            // true -> nothing was selected
            if(_nbOfItems != 0
                && MochiKit.Iter.every(
                        _values,
                        function(item)
                        {
                            return MochiKit.Format.strip(item) == ""
                                || MochiKit.Base.isUndefinedOrNull(item);
                        }
                    )
                )
            {
                // prepare message
                var _message = "Field could not be empty!";
                // append warnig message
                BewypeJS.Utils.Widget.appendMessage(
                        elmtToValidate, "Warning", _message);
                //
                return false;
            }
            else
            {
                return _values;
            }
        }
        else
        {
            return _values;
        }
    },

    /**
     * Common empty field validator method. Check if value exist and if is empty.
     * Append warning message with appendMessage function.
     */
    validateNotEmpty: function (elmtToValidate)
    {
        if(MochiKit.Base.isUndefinedOrNull(elmtToValidate))
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Validators] validateNotEmpty - elmtToValidate is NULL for: "
                    + elmtToValidate.id);
            // DEBUG
            return false;
        }
        else
        {
            // continue
            MochiKit.Base.noop();
        }

        // get value to check
        var _value = BewypeJS.Utils.getFieldValue(elmtToValidate);

        if(MochiKit.Base.isUndefinedOrNull(_value)
            || _value == "")
        {
            // prepare message
            var _message = "Field could not be empty!";
            // append warnig message
            BewypeJS.Utils.Widget.appendMessage(elmtToValidate, "Warning", _message);
            //
            return false;
        }
        else
        {
            return true;
        }
    },

    /**
     * Common email field validator method. Check if value exist and
     * value format is valid.
     *
     * Append warning message with appendMessage function.
     */
    validateEmail: function (elmtToValidate)
    {
        // get non empty value
        var _isNotEmpty = BewypeJS.Validators.validateNotEmpty(elmtToValidate);

        // check if exist
        if(_isNotEmpty == false)
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Validators] validateEmail - found empty value for: "
                    + elmtToValidate.id);
            // DEBUG
            return false;
        }
        else
        {
            // get value to check
            var _value = BewypeJS.Utils.getFieldValue(elmtToValidate);
            // prepare regex
            var _emailRegEx = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i;
            // check the email format
            if (_value.search(_emailRegEx) == -1)
            {
                // prepare message
                var _message = "Field has an invalid email format!";
                // append warnig message
                BewypeJS.Utils.Widget.appendMessage(elmtToValidate, "Warning", _message);
                // return false
                return false;
            }
            else
            {
                return true;
            }
        }
    },

    /**
     *
     */
    validateDate: function (elmtToValidate, dateFormat)
    {
        // get non empty value
        var _isNotEmpty = BewypeJS.Validators.validateNotEmpty(elmtToValidate);

        // check if exist
        if(_isNotEmpty == false)
        {
            // DEBUG
            MochiKit.Logging.logDebug(
                    "[Validators] validateDate - found empty value for: "
                    + elmtToValidate.id);
            // DEBUG
            return false;
        }
        else
        {
            // get value to check
            var _value = BewypeJS.Utils.getFieldValue(elmtToValidate);
            // should not throw an exception with a valid format
            var _dateObj = Other.DateFormat.parseDate(_value, dateFormat);
            // 
            if (MochiKit.Base.isUndefinedOrNull(_dateObj))
            {
                // Oops! prepare message
                var _message = "Field has an invalid date format!";
                // append warnig message
                BewypeJS.Utils.Widget.appendMessage(elmtToValidate, "Warning", _message);
                return false;
            }
            else
            {
                return true;
            }
        }
    }
}

