#!/usr/bin/env python
# -*- coding: utf-8 -*-

# python import
import os

# get pypi doc
f = open(os.path.join(os.path.dirname(__file__),
            "docs", "sphinx", "doc", "intro.rst"))
_long_description = f.read().strip()
f.close()

# setuptools import
from setuptools import setup, find_packages

# bewype import
from bewype import __version__ as version

setup(
    name="bewype",
    version=version,
    author="Florent Pigout",
    author_email="florent.pigout@bewype.org",
    description="Bewype widget based web application framework core.",
    long_description=_long_description,
    license = "MIT",
    url="http://www.bewype.org",
    packages=find_packages(exclude=[]),
    package_data = {
        "": ["*/*.*-tmpl"],
        "bewype.db.repository": ["migrate.cfg"],
        "bewype.js.resource.css": ["*/*.css"],
        "bewype.js.resource.img": ["*/*.gif", "*/*.jpg", "*/*.png"],
        "bewype.js.resource.script": ["*/*.js", "*/__modules"],
        },
    data_files=["COPYING"],
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        "Babel>=0.9.5",
        "Genshi>=0.5.1",
        "Importing>=1.9.2",
        "zope.interface>=3.6.1",
        "Twisted>=9.0.0",
        "Nevow>=0.9.33",
        "pyOpenSSL>=0.10",
        "PyYAML>=3.09",
        "recaptcha-client>=1.0.5",
        "simplejson>=2.0.9",
        "sqlalchemy-migrate>=0.5.4",
        "TurboMail>=3.0.3",
        ],
    namespace_packages=[
            "bewype",
            "bewype.db",
            "bewype.core",
            "bewype.js",
            ],
    entry_points={
        "console_scripts": [
            "bewype_core_doctest=bewype.core.tests.main:run_doctest"
            ],
        },
    test_suite = "bewype.core.tests",
    )

